##
## runner.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Thu May 17 22:02:08 2007 David Rousselie
## $Id$
##
## Copyright (C) 2007 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import logging
import os
import sys
from ConfigParser import ConfigParser
from getopt import gnu_getopt

from jcl.lang import Lang
from jcl.jabber.component import JCLComponent
import jcl.model as model
from jcl.model.account import Account, PresenceAccount, User, LegacyJID

class JCLRunner(object):

    def __init__(self, component_name, component_version):
        """
        options: list of tuples:
        - short_opt: same as getopt
        - long_opt: same as getopt
        """
        self.component_short_name = "JCL"
        self.component_name = component_name
        self.component_version = component_version
        self.config_file = "jcl.conf"
        self.server = "localhost"
        self.port = 5347
        self.secret = "secret"
        self.service_jid = "jcl.localhost"
        self.language = "en"
        self.db_url = "sqlite:///var/spool/jabber/jcl.db"
        self.pid_file = "/var/run/jabber/jcl.pid"
        self.log_stdout = False
        self.log_file = None
        self.console = False
        self.options = [("c:", "config-file=", None,
                         " FILE\t\t\t\tConfiguration file to use",
                         lambda arg: self.set_attr("config_file", arg)),
                        ("S:", "server=", "jabber",
                         " SERVER_ADDRESS\t\t\tAddress of the Jabber server",
                         lambda arg: self.set_attr("server", arg)),
                        ("P:", "port=", "jabber",
                         " PORT\t\t\t\t\tPort of the Jabber server to connect the component",
                         lambda arg: self.set_attr("port", int(arg))),
                        ("s:", "secret=", "jabber",
                         " SECRET\t\t\t\tComponent password to connect to the Jabber server",
                         lambda arg: self.set_attr("secret", arg)),
                        ("j:", "service-jid=", "jabber",
                         " JID\t\t\t\tJID of the component",
                         lambda arg: self.set_attr("service_jid", arg)),
                        ("l:", "language=", "jabber",
                         " LANG\t\t\t\tDefault Language of the component",
                         lambda arg: self.set_attr("language", arg)),
                        ("u:", "db-url=", "db",
                         " URL\t\t\t\tDatabase URL",
                         lambda arg: self.set_attr("db_url", arg)),
                        ("p:", "pid-file=", "component",
                         " FILE\t\t\t\tPath of the PID file",
                         lambda arg: self.set_attr("pid_file", arg)),
                        ("d", "debug", None,
                         "\t\t\t\t\tEnable debug traces",
                         lambda arg: self.set_attr("debug", True)),
                        ("C", "console", None,
                         "\t\t\t\t\tRun in console mode",
                         lambda arg: self.set_attr("console", True)),
                        ("o", "log-stdout", None,
                         "\t\t\t\t\tLog on stdout",
                         lambda arg: self.set_attr("log_stdout", True)),
                        ("f", "log-file", "component",
                         "\t\t\t\t\tLog in file",
                         lambda arg: self.set_attr("log_file", arg)),
                        ("h", "help", None,
                         "\t\t\t\t\tThis help",
                         lambda arg: self.print_help())]
        self.logger = logging.getLogger()
        self.__debug = False

    def set_attr(self, attr, value):
        setattr(self, attr, value)

    def __configure_commandline_args(self, shortopts, longopts, cleanopts):
        (opts, args) = gnu_getopt(sys.argv[1:], shortopts, longopts)
        commandline_args = {}
        for (arg, value) in opts:
            clean_arg = arg.lstrip('-')
            if cleanopts.has_key(clean_arg):
                commandline_args[clean_arg] = value
        return commandline_args

    def __apply_commandline_args(self, commandline_args, cleanopts):
        for arg in commandline_args:
            value = commandline_args[arg]
            self.logger.debug("Applying argument " + arg + " = " +
                              value)
            cleanopts[arg][1](value)

    def __apply_configfile(self, commandline_args, cleanopts):
        if commandline_args.has_key("config_file"):
            self.config_file = commandline_args["config_file"]
        elif commandline_args.has_key("c"):
            self.config_file = commandline_args["c"]
        self.config = ConfigParser()
        self.logger.debug("Loading config file " + self.config_file)
        read_file = self.config.read(self.config_file)
        if read_file == []:
            self.logger.info("Creating empty config file " + self.config_file)
        else:
            for opt in cleanopts:
                if len(opt) > 1:
                    (section, set_func) = cleanopts[opt]
                    if section is not None:
                        attr = opt.replace("-", "_")
                        config_property = self.config.get(section, attr)
                        self.logger.debug("Setting " + attr + " = " +
                                          config_property +
                                          " from configuration file " +
                                          self.config_file)
                        set_func(config_property)

    def configure(self):
        """
        Apply configuration from command line and configuration file.
        command line arguments override configuration file properties.
        """
        shortopts = ""
        longopts = []
        cleanopts = {}
        for option in self.options:
            shortopts += option[0]
            longopts += [option[1]]
            cleanopts[option[0][0]] = (option[2], option[4])
            if option[1][-1:] == '=':
                cleanopts[option[1][:-1]] = (option[2], option[4])
            else:
                cleanopts[option[1]] = (option[2], option[4])

        commandline_args = self.__configure_commandline_args(shortopts, longopts, cleanopts)
        if commandline_args.has_key("debug") or commandline_args.has_key("d"):
            self.debug = True
            self.logger.debug("Debug activated")
        self.__apply_configfile(commandline_args, cleanopts)
        self.__apply_commandline_args(commandline_args, cleanopts)
        if self.log_stdout:
            self.logger.addHandler(logging.StreamHandler())
        if self.log_file is not None:
            self.logger.addHandler(logging.FileHandler(self.log_file))

    def _get_help(self):
        help = self.component_name + " v" + self.component_version + " help:\n"
        for option in self.options:
            if option[1][-1:] == '=':
                long_option = option[1][:-1]
            else:
                long_option = option[1]
            help += "\t-" + option[0][0] + ", --" + long_option + option[3] + "\n"
        return help

    def print_help(self):
        print self._get_help()
        sys.exit(0)

    def get_debug(self):
        return self.__debug

    def set_debug(self, value):
        self.__debug = value
        if self.__debug:
            self.logger.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.CRITICAL)

    debug = property(get_debug, set_debug)


    def setup_db(self):
        Account.createTable(ifNotExists=True)
        PresenceAccount.createTable(ifNotExists=True)
        User.createTable(ifNotExists=True)
        LegacyJID.createTable(ifNotExists=True)

    def setup_pidfile(self):
        pidfile = open(self.pid_file, "w")
        pidfile.write(str(os.getpid()))
        pidfile.close()

    def _run(self, run_func):
        if self.console:
            self.run_console()
        else:
            try:
                self.setup_pidfile()
                model.db_connection_str = self.db_url
                model.db_connect()
                self.setup_db()
                model.db_disconnect()
                self.logger.debug(self.component_name + " v" +
                                  self.component_version + " is starting ...")
                restart = True
                while restart:
                    restart = run_func()
                self.logger.debug(self.component_name + " is exiting")
            finally:
                if os.path.exists(self.pid_file):
                    os.remove(self.pid_file)

    def run(self):
        def run_func():
            component = JCLComponent(jid=self.service_jid,
                                     secret=self.secret,
                                     server=self.server,
                                     port=self.port,
                                     lang=Lang(self.language),
                                     config=self.config,
                                     config_file=self.config_file)
            return component.run()
        self._run(run_func)

    def run_console(self):
        """
        Run JCL console. Used to access SQLObject connection
        """
        from IPython.Shell import IPShellEmbed
        # pre-import jcl.model.account to be used in the shell
        import jcl.model.account as account
        model.db_connection_str = self.db_url
        model.db_connect()
        self.setup_db()
        ipshell = IPShellEmbed(["-pi1", self.component_short_name + "[\\#]: "], 
                               "Starting " + self.component_name + " v" \
                                   + self.component_version + " console.",
                               "Ending " + self.component_name + " v" \
                                   + self.component_version + " console.")
        ipshell()

def main():
    import sys
    reload(sys)
    sys.setdefaultencoding('utf-8')
    del sys.setdefaultencoding
    import jcl
    from jcl.runner import JCLRunner
    runner = JCLRunner("JCL", jcl.version)
    runner.configure()
    runner.run()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
##
## feeder.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Aug  9 21:04:42 2006 David Rousselie
## $Id$
## 
## Copyright (C) 2006 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

"""FeederComponent with default Feeder and Sender
implementation"""

__revision__ = "$Id: feeder.py,v 1.3 2005/09/18 20:24:07 dax Exp $"

import logging

from jcl.jabber import Handler
from jcl.jabber.component import JCLComponent, AccountManager
from jcl.lang import Lang
import jcl.model as model
from jcl.model import account
from jcl.jabber.command import JCLCommandManager

from pyxmpp.message import Message

class FeederComponent(JCLComponent):
    """
    Implement a feeder sender behavior based on the
    regular interval behavior of JCLComponent
    feed data from given Feeder and send it to user
    through the given Sender.
    """

    def __init__(self,
                 jid,
                 secret,
                 server,
                 port,
                 config,
                 config_file,
                 lang=Lang(),
                 account_manager_class=AccountManager,
                 command_manager_class=JCLCommandManager):
        JCLComponent.__init__(self,
                              jid,
                              secret,
                              server,
                              port,
                              config,
                              config_file,
                              lang=lang,
                              account_manager_class=account_manager_class,
                              command_manager_class=command_manager_class)
        # Define default feeder and sender, can be override
        self.handler = FeederHandler(Feeder(self), Sender(self))
        self.check_interval = 1

        self.__logger = logging.getLogger("jcl.jabber.FeederComponent")
        
    def handle_tick(self):
        """Implement main feed/send behavior"""
        model.db_connect()
        self.handler.handle(\
            None, self.lang.get_default_lang_class(), 
            self.handler.filter(None, 
                                self.lang.get_default_lang_class()))
        model.db_disconnect()



class Feeder(object):
    """Abstract feeder class"""
    def __init__(self, component = None):
        self.component = component

    def feed(self, _account):
        """Feed data for given account"""
        raise NotImplementedError


class Sender(object):
    """Abstract sender class"""
    def __init__(self, component = None):
        self.component = component

    def send(self, to_account, subject, body):
        """Send data (subject and body) to given account"""
        raise NotImplementedError

class MessageSender(Sender):
    """Send data as Jabber Message"""

    def create_message(self, to_account, data):
        """Create message to send"""
        subject, body = data
        return Message(from_jid=to_account.jid,
                       to_jid=to_account.user.jid,
                       subject=subject,
                       body=body)

    def send(self, to_account, data):
        """Implement abstract method from Sender class and send
        data as Jabber message.
        """
        self.component.stream.send(self.create_message(to_account,
                                                       data))

class HeadlineSender(MessageSender):
    """Send data as Jabber Headline"""

    def create_message(self, to_account, data):
        """Create headline to send"""
        subject, body = data
        return Message(from_jid=to_account.jid,
                       to_jid=to_account.user.jid,
                       subject=subject,
                       stanza_type="headline",
                       body=body)

class FeederHandler(Handler):
    """
    Filter (nothing by default) and call sender for each message from
    Feeder.
    """

    def __init__(self, feeder, sender):
        """DefaultFeederHandler constructor"""
        self.feeder = feeder
        self.sender = sender

    def filter(self, stanza, lang_class):
        """
        Filter account to be processed by the handler
        return all accounts.
        """
        accounts = account.get_all_accounts()
        return accounts

    def handle(self, stanza, lang_class, data):
        """
        Apply actions to do on given accounts
        Do nothing by default.
        """
        for _account in data:
            if _account.enabled:
                for data in self.feeder.feed(_account):
                    self.sender.send(_account, data)
        return []




##
## error.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Sun Nov  5 20:13:48 2006 David Rousselie
## $Id$
## 
## Copyright (C) 2006 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

"""Jabber exception classes"""

__revision__ = "$Id: error.py,v 1.1 2006/11/05 20:13:48 dax Exp $"

class FieldError(Exception):
    """Error raised when error exists on Jabber Data Form fields"""

    def __init__(self, field, error_msg):
        Exception.__init__(self)
        self.field = field
        self.error_msg = error_msg

    def __str__(self):
        return "Error with " + str(self.field) + " field: " + str(self.error_msg)



##
## message.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Jun 27 21:43:57 2007 David Rousselie
## $Id$
##
## Copyright (C) 2007 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import re

from pyxmpp.message import Message

from jcl.jabber import Handler
import jcl.model.account as account
from jcl.model.account import Account

class PasswordMessageHandler(Handler):
    """Handle password message"""

    def __init__(self, component):
        """handler constructor"""
        Handler.__init__(self, component)
        self.password_regexp = re.compile("\[PASSWORD\]")

    def filter(self, stanza, lang_class):
        """
        Return the uniq account associated with a name and user JID.
        DB connection might already be opened.
        """
        _account = account.get_account(unicode(stanza.get_from().bare()),
                                       stanza.get_to().node)
        if hasattr(_account, 'password') \
               and hasattr(_account, 'waiting_password_reply') \
               and (getattr(_account, 'waiting_password_reply') == True) \
               and self.password_regexp.search(stanza.get_subject()) \
               is not None:
            return _account
        else:
            return None

    def handle(self, stanza, lang_class, data):
        """
        Receive password in stanza (must be a Message) for given account.
        """
        return self.component.account_manager.set_password(data, stanza.get_from(),
                                                           stanza.get_body(),
                                                           lang_class)

class HelpMessageHandler(Handler):
    """Handle 'help' sent in a message"""

    def __init__(self, component):
        """Handler constructor"""
        Handler.__init__(self, component)
        self.help_regexp = re.compile("^help.*")

    def filter(self, stanza, lang_class):
        """
        Test if stanza body match the help regexp.
        """
        return self.help_regexp.search(stanza.get_body()) \
            or self.help_regexp.search(stanza.get_subject())

    def handle(self, stanza, lang_class, data):
        """
        Return a help message.
        """
        return [Message(to_jid=stanza.get_from(),
                        from_jid=stanza.get_to(),
                        stanza_type=stanza.get_type(),
                        subject=lang_class.help_message_subject,
                        body=lang_class.help_message_body)]



##
## disco.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Jun 27 22:27:25 2007 David Rousselie
## $Id$
##
## Copyright (C) 2007 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import logging

from pyxmpp.jid import JID
from pyxmpp.jabber.disco import DiscoInfo, DiscoItems, DiscoItem, DiscoIdentity

import jcl.jabber as jabber

class DiscoHandler(object):
    """Handle disco get items requests"""

    def __init__(self, component):
        self.component = component

    def filter(self, stanza, lang_class, node):
        """Filter requests to be handled"""
        return False

    def handle(self, stanza, lang_class, node, disco_obj, data):
        """Handle disco get items request"""
        return None

class RootDiscoGetInfoHandler(DiscoHandler):

    filter = jabber.root_filter

    def __init__(self, component):
        DiscoHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.RootDiscoGetInfoHandler")

    def handle(self, stanza, lang_class, node, disco_obj, data):
        """Implement discovery get_info on main component JID"""
        self.__logger.debug("root_disco_get_info")
        disco_info = DiscoInfo(node)
        disco_info.add_feature("jabber:iq:version")
        disco_info.add_feature("http://jabber.org/protocol/disco#info")
        disco_info.add_feature("http://jabber.org/protocol/disco#items")
        if not self.component.account_manager.has_multiple_account_type:
            disco_info.add_feature("jabber:iq:register")
        DiscoIdentity(disco_info, self.component.name,
                      self.component.disco_identity.category,
                      self.component.disco_identity.type)
        return [disco_info]

class AccountDiscoGetInfoHandler(DiscoHandler):

    filter = jabber.account_filter

    def __init__(self, component):
        DiscoHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.AccountDiscoGetInfoHandler")

    def handle(self, stanza, lang_class, node, disco_obj, data):
        """Implement discovery get_info on an account node"""
        self.__logger.debug("account_disco_get_info")
        disco_info = DiscoInfo(node)
        disco_info.add_feature("jabber:iq:register")
        return [disco_info]

class AccountTypeDiscoGetInfoHandler(AccountDiscoGetInfoHandler):

    filter = jabber.account_type_filter

    def __init__(self, component):
        AccountDiscoGetInfoHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.AccountTypeDiscoGetInfoHandler")

class RootDiscoGetItemsHandler(DiscoHandler):

    filter = jabber.root_filter

    def __init__(self, component):
        DiscoHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.RootDiscoGetItemsHandler")
        
    def handle(self, stanza, lang_class, node, disco_obj, data):
        """Discovery get_items on root node"""
        from_jid = stanza.get_from()
        if node is not None:
            return None
        disco_items = None
        if self.component.account_manager.has_multiple_account_type: # list accounts with only one type declared
            disco_items = DiscoItems(node)
            for (account_type, type_label) in \
                    self.component.account_manager.list_account_types(lang_class):
                DiscoItem(disco_items,
                          JID(unicode(self.component.jid) + "/" +
                              account_type),
                          account_type,
                          type_label)

        else:
            disco_items = DiscoItems(node)
            for (_account, resource, account_type) in \
                    self.component.account_manager.list_accounts(unicode(from_jid.bare())):
                DiscoItem(disco_items,
                          JID(unicode(_account.jid) + resource),
                          account_type + _account.name,
                          _account.long_name)
        return [disco_items]

class AccountTypeDiscoGetItemsHandler(DiscoHandler):

    filter = jabber.account_type_filter

    def __init__(self, component):
        DiscoHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.AccountTypeDiscoGetItemsHandler")
        
    def handle(self, stanza, lang_class, node, disco_obj, data):
        """Discovery get_items on an account type node"""
        account_type = data
        from_jid = stanza.get_from()
        self.__logger.debug("Listing account for " + account_type)
        account_class = self.component.account_manager.get_account_class(account_type)
        if account_class is not None:
            disco_items = DiscoItems(node)
            for (_account, resource, account_type) in \
                    self.component.account_manager.list_accounts(unicode(from_jid.bare()),
                                                                 account_class,
                                                                 account_type=account_type):
                DiscoItem(disco_items,
                          JID(unicode(_account.jid) + resource),
                          account_type + _account.name,
                          _account.long_name)
            return [disco_items]
        else:
            self.__logger.error("Error: " + str(account_class)
                                + " class not in account_classes")
            return []



"""Contains data model classes"""
__revision__ = ""

from sqlobject.dbconnection import ConnectionHub
from sqlobject.dbconnection import connectionForURI

import jcl.model

db_connection_str = ""

# create a hub to attach a per thread connection
hub = ConnectionHub()

def db_connect():
    """
    Create a new connection to the DataBase (SQLObject use connection
    pool) associated to the current thread.
    """
    #if not jcl.model.db_connected:
    jcl.model.hub.threadConnection = \
        connectionForURI(db_connection_str)
    #        account.hub.threadConnection.debug = True
    #jcl.model.db_connected = True

def db_disconnect():
    """
    Delete connection associated to the current thread.
    """
    pass
    #if jcl.model.db_connected:
    #del jcl.model.hub.threadConnection
    #jcl.model.db_connected = False



"""Jabber related classes"""
__revision__ = ""

import jcl.model.account as account
from jcl.model.account import Account

class Handler(object):
    """handling class"""

    def __init__(self, component):
        """Default Handler constructor"""
        self.component = component

    def filter(self, stanza, lang_class):
        """
        Filter account to be processed by the handler
        return all accounts. DB connection might already be opened.
        """
        accounts = account.get_all_accounts()
        return accounts

    def handle(self, stanza, lang_class, data):
        """
        Apply actions to do on given accounts
        Do nothing by default.
        """
        return []

def root_filter(self, stanza, lang_class, node=None):
    """Filter stanza sent to root node"""
    to_jid = stanza.get_to()
    if to_jid.resource is None and to_jid.node is None and node is None:
        return True
    else:
        return None

def account_type_filter(self, stanza, lang_class, node=None):
    """Filter stanzas sent to account type node"""
    to_jid = stanza.get_to()
    account_type = to_jid.resource
    if account_type is not None and to_jid.node is None:
        return account_type
    else:
        return None

def account_filter(self, stanza, lang_class, node=None):
    """Filter stanzas sent to account jid"""
    name = stanza.get_to().node
    return name

def get_account_filter(self, stanza, lang_class, node=None):
    """Filter stanzas sent to account jid, only if account exists"""
    name = stanza.get_to().node
    if name is not None:
        return account.get_account(unicode(stanza.get_from().bare()),
                                   name)
    else:
        return None

def get_accounts_root_filter(self, stanza, lang_class, node=None):
    """Filter stanza sent to root node"""
    to_jid = stanza.get_to()
    if to_jid.resource is None and to_jid.node is None and node is None:
        return account.get_accounts(unicode(stanza.get_from().bare()))
    else:
        return None

def replace_handlers(handlers, old_handler_type, new_handler):
    """
    Replace handlers of type `old_handler_type` in `handlers` by `new_handler`
    """
    for handler_group in handlers:
        for i in xrange(len(handler_group)):
            if handler_group[i].__class__.__name__ == old_handler_type.__name__:
                handler_group[i] = new_handler



# -*- coding: utf-8 -*-
##
## lang.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Sat Jan 28 16:37:11 2006 David Rousselie
## $Id$
##
## Copyright (C) 2006 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

"""
lang -- contains translations
"""

__revision__ = "$Id: lang.py,v 1.3 2005/09/18 20:24:07 dax Exp $"

class Lang:
    """
    Lang.
    """

    # pylint: disable-msg=W0232, R0903, C0103, C0111
    def __init__(self, default_lang = "en"):
        self.default_lang = default_lang

    def get_lang_from_node(self, node):
        """
        Extract lang contain in a XML node.

        :Parameters:
           - `node`: XML node.
        """
        lang = node.getLang()
        if lang is None:
            lang = self.default_lang
        return lang

    def get_lang_class(self, lang):
        """Return lang class from lang code.

        :Parameters:
           - `lang`: lang code.
        """
        if lang is not None:
            lang = lang[:2]
        if hasattr(self.__class__, lang):
            return getattr(self.__class__, lang)
        return getattr(self.__class__, self.default_lang)

    def get_lang_class_from_node(self, node):
        """Return lang class from XML node.

        :Parameters:
           - `node`: XML node.
        """
        return self.get_lang_class(self.get_lang_from_node(node))

    def get_default_lang_class(self):
        """Return default lang class according to self.default_lang"""
        return self.get_lang_class(self.default_lang)

    class en(object):
        component_name = u"Jabber Component Library generic component"
        register_title = u"Create new account"
        register_instructions = u"Enter account parameters"
        message_status = u" accounts registered."
        account_name = u"Account name"

        password_saved_for_session = u"Password will be kept during your " \
                                     u"Jabber session"
        ask_password_subject = u"Password request"
        ask_password_body = u"Reply to this message with the password " \
                            "for account %s\n"
        new_account_message_subject = u"New account '%s' created"
        new_account_message_body = u"New account created"
        update_account_message_subject = u"Updated account '%s'"
        update_account_message_body = u"Updated account"

        mandatory_field = u"%s is required"

        field_chat_action = u"Action when state is 'Free For Chat'"
        field_online_action = u"Action when state is 'Online'"
        field_away_action = u"Action when state is 'Away'"
        field_xa_action = u"Action when state is 'Not Available'"
        field_dnd_action = u"Action when state is 'Do not Disturb'"
        field_offline_action = u"Action when state is 'Offline'"

        field_action_0 = u"Do nothing"
        field_chat_action_0 = field_action_0
        field_online_action_0 = field_action_0
        field_away_action_0 = field_action_0
        field_xa_action_0 = field_action_0
        field_dnd_action_0 = field_action_0
        field_offline_action_0 = field_action_0

        field_user_jid = u"User Jabber ID"
        field_password = u"Password"

        error_subject = u"Error"
        error_body = u"An error has occured:\n\t%s"

        get_gateway_desc = u"Please enter the email address of your contact"
        get_gateway_prompt = u"Email address"

        command_add_user = u"Create new account"
        command_add_user_1_description = u"Select an account type"
        field_account_type = u"Account type"

        command_delete_user = u"Delete accounts"
        command_delete_user_1_description = \
            u"Fill out this form with user(s) Jabber ID(s) to search account(s) to delete"
        command_delete_user_2_description = u"Select account(s) to delete"
        field_users_jids = u"User(s) Jabber ID(s)"
        field_account = u"Account"
        field_accounts = u"Accounts"

        command_disable_user = u"Disable accounts"
        command_disable_user_1_description = \
            u"Fill out this form with user(s) Jabber ID(s) to search account(s) to disable"
        command_disable_user_2_description = u"Select account(s) to disable"

        command_reenable_user = u"Re-Enable accounts"
        command_reenable_user_1_description = \
            u"Fill out this form with user(s) Jabber ID(s) to search account(s) to re-enable"
        command_reenable_user_2_description = u"Select account(s) to re-enable"

        command_end_user_session = u"End users sessions"
        command_end_user_session_1_description = \
            u"Fill out this form with user(s) Jabber ID(s) to search account(s) to end session"
        command_end_user_session_2_description = \
            u"Select account(s) to end their session"

        command_get_user_password = u"Get user&apos;s password"
        command_get_user_password_1_description = \
            u"Fill out this form with user Jabber ID to search account" \
            u" for which to retrieve password"
        command_get_user_password_2_description = \
            u"Select account for which to retrieve password"

        command_change_user_password = u"Changing user&apos;s password"
        command_change_user_password_1_description = \
            u"Fill out this form with user Jabber ID to search account" \
            u" for which to change password"
        command_change_user_password_2_description = \
            u"Select account for which to change password"

        command_get_user_roster = u"Get user's accounts"
        command_get_user_roster_1_description = \
            u"Fill out this form with user Jabber ID for which to retrieve accounts"

        command_get_user_lastlogin = u"Get user's last login time"
        command_get_user_lastlogin_1_description = \
            u"Fill out this form with user Jabber ID to search account" \
            u" for which to retrieve last login time"
        command_get_user_lastlogin_2_description = \
            u"Select account for which to retrieve last login time"

        command_get_registered_users_num = u"Get the number of " \
                                           u"registered accounts"
        command_get_disabled_users_num = u"Get the number of disabled accounts"
        command_get_online_users_num = u"Get the number of online accounts"

        command_get_registered_users_list = u"Get the list of " \
                                           u"registered accounts"
        command_get_disabled_users_list = u"Get the list of disabled accounts"
        command_get_online_users_list = u"Get the list of online accounts"

        field_registered_users_num = u"The number of registered accounts"
        field_disabled_users_num = u"The number of disabled accounts"
        field_online_users_num = u"The number of online accounts"

        field_max_items = u"Maximum number of items to show"
        field_registered_users_list = u"The list of registered accounts"
        field_disabled_users_list = u"The list of disabled accounts"
        field_online_users_list = u"The list of online accounts"

        command_announce = u"Make an annoucement"
        command_announce_1_description = \
            u"Fill out this form with the announce to send to online users"
        field_announcement = u"Announcement"

        command_set_motd = u"Set the Message Of The Day"
        command_set_motd_1_description = \
            u"Fill out this form with the Message Of The Day to send to all" \
            u" registered users"
        field_motd = u"Message Of The Day"

        command_edit_motd = u"Edit the Message Of The Day"

        command_delete_motd = u"Delete the Message Of The Day"

        command_set_welcome = u"Set Welcome Message"
        command_set_welcome_1_description = \
            u"Fill out this form with the Welcome Message to send to all" \
            u" new registered users"
        field_welcome = u"Welcome Message"

        command_delete_welcome = u"Delete Welcome Message"

        command_edit_admin = u"Edit the admin list"
        command_edit_admin_1_description = \
            u"Fill out this form to edit the list of users who have " \
            u"administrative privileges."
        field_admin_jids = u"The admin list"

        command_restart = u"Restart the service"
        command_restart_1_description = \
            u"Fill out this form to restart the service."
        field_restart_delay = u"Delay before restarting the service"

        command_shutdown = u"Shut down the service"
        command_shutdown_1_description = \
            u"Fill out this form to shut down the service."
        field_shutdown_delay = u"Delay before shutting down the service"

        field_30_sec = u"30 seconds"
        field_60_sec = u"60 seconds"
        field_90_sec = u"90 seconds"
        field_120_sec = u"120 seconds"
        field_180_sec = u"180 seconds"
        field_240_sec = u"240 seconds"
        field_300_sec = u"300 seconds"

        welcome_message_subject = u"Welcome"

        account_disabled = u"This account is disabled"
        account_error = u"This account has an error"

        help_message_subject = u"Help"
        help_message_body = u"No help"

        command_last_error = u"Get account's last error"
        field_last_error = u"Last error"
        account_no_error = u"This account has no error"

    class fr(en):
        component_name = u"composant générique Jabber Component Library"
        register_title = u"Enregistrement d'un nouveau compte"
        register_instructions = u"Entrer les paramètres du compte"
        message_status = u" comptes enregistrés."
        account_name = u"Nom du compte"

        password_saved_for_session = u"Le mot de passe sera garder tout au " \
                                     u"long de la session Jabber."
        ask_password_subject = u"Demande de mot de passe"
        ask_password_body = u"Répondre à ce message avec le mot de passe " \
                            u"du compte %s\n"
        new_account_message_subject = u"Le compte '%s' a été créé"
        new_account_message_body = u"Compte créé"
        update_account_message_subject = u"Le compte '%s' a été mis " \
                                         u"à jour"
        update_account_message_body = u"Compte mis à jour"

        mandatory_field = u"%s est requis"

        field_chat_action = u"Action lorsque l'état est 'Free For Chat'"
        field_online_action = u"Action lorsque l'état est 'Online'"
        field_away_action = u"Action lorsque l'état est 'Away'"
        field_xa_action = u"Action lorsque l'état est 'Not Available'"
        field_dnd_action = u"Action lorsque l'état est 'Do not Disturb'"
        field_offline_action = u"Action lorsque l'état est 'Offline'"

        field_action_0 = u"Ne rien Faire"
        field_chat_action_0 = field_action_0
        field_online_action_0 = field_action_0
        field_away_action_0 = field_action_0
        field_xa_action_0 = field_action_0
        field_dnd_action_0 = field_action_0
        field_offline_action_0 = field_action_0

        field_user_jid = u"JID utilisateur"
        field_password = u"Mot de passe"

        error_subject = u"Erreur"
        error_body = u"Une erreur est survenue :\n\t%s"

        get_gateway_desc = u"Entrer l'adresse email de votre contact"
        get_gateway_prompt = u"Adresse email"

        command_add_user = u"Créer un nouveau compte"
        command_add_user_1_description = u"Selectionner un type de compte"
        field_account_type = u"Type de compte"

        command_delete_user = u"Supprimer des comptes"
        command_delete_user_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s) pour" \
            u" rechercher le(s) compte(s) à supprimer"
        command_delete_user_2_description = u"Sélectionner le(s) compte(s) à " \
            u"supprimer"
        field_users_jids = u"Jabber ID(s) utilisateur(s)"
        field_account = u"Compte"
        field_accounts = u"Comptes"

        command_disable_user = u"Désactiver des comptes"
        command_disable_user_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s) " \
            u"pour rechercher le(s) compte(s) à désactiver"
        command_disable_user_2_description = u"Sélectionner le(s) compte(s) à" \
                                             u" désactiver"

        command_reenable_user = u"Réactiver des comptes"
        command_reenable_user_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s) " \
            u"pour rechercher le(s) compte(s) à réactiver"
        command_reenable_user_2_description = u"Sélectionner le(s) compte(s)" \
                                              u" à réactiver"

        command_end_user_session = u"Terminer des sessions utilisateurs"
        command_end_user_session_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s)" \
            u" pour rechercher le(s) session(s) à terminer"
        command_end_user_session_2_description = \
            u"Sélectionner le(s) compte(s) des sessions à terminer"

        command_get_user_password = u"Récupérer le mot de passe d'un compte"
        command_get_user_password_1_description = \
            u"Remplir ce formulaire avec le Jabber ID utilisateur" \
            u" pour rechercher le compte pour récupérer leur mot de passe"
        command_get_user_password_2_description = \
            u"Sélectionner le compte pour récupérer leur mot de passe"

        command_change_user_password = u"Changer le mot de passe d'un compte"
        command_change_user_password_1_description = \
            u"Remplir ce formulaire avec le Jabber ID utilisateur" \
            u" pour rechercher le compte pour changer leur mot de passe"
        command_change_user_password_2_description = \
            u"Sélectionner le compte pour changer leur mot de passe"

        command_get_user_roster = u"Récupérer les comptes utilisateurs"
        command_get_user_roster_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s)" \
            u" pour rechercher le(s) compte(s) à récupérer"

        command_get_user_lastlogin = u"Récupérer la date de derniere connexion"
        command_get_user_lastlogin_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s)" \
            u" pour rechercher le(s) compte(s) pour récupérer leur dernière " \
            u"date de connexion"
        command_get_user_lastlogin_2_description = \
            u"Sélectionner le compte pour récupérer la date de dernière connexion"

        command_get_registered_users_num = u"Récupérer de nombre de comptes " \
                                           u"enregistrés"
        command_get_disabled_users_num = u"Récupérer le nombre de comptes " \
                                         u"désactivés"
        command_get_online_users_num = u"Récupérer le nombre de comptes " \
                                       u"en ligne"

        command_get_registered_users_list = u"Récupérer la liste des comptes " \
                                           u"enregistrés"
        command_get_disabled_users_list = u"Récupérer la list des comptes " \
                                          u"désactivés"
        command_get_online_users_list = u"Récupérer la list des comptes " \
                                        u"en ligne"

        field_registered_users_num = u"Le nombre de comptes enregistrés"
        field_disabled_users_num = u"Le nombre de comptes désactivés"
        field_online_users_num = u"Le nombre de comptes connectés"

        field_max_items = u"Maximum d'éléments à récupérer"
        field_registered_users_list = u"La liste des comptes enregistrés"
        field_disabled_users_list = u"La liste des comptes désactivés"
        field_online_users_list = u"La liste des comptes connectés"

        command_announce = u"Faire une annonce"
        command_announce_1_description = \
            u"Remplir ce formulaire avec le(s) Jabber ID(s) utilisateur(s)" \
            u" à qui envoyer l'annonce"
        field_announcement = u"Annonce"

        command_set_motd = u"Écrire le Message Du Jour"
        command_set_motd_1_description = \
            u"Remplir ce formulaire avec le Message Du Jour à envoyer à tous" \
            u" les utilisateurs enregistrés"
        field_motd = u"Message Du Jour"

        command_edit_motd = u"Editer le Message Du Jour"

        command_delete_motd = u"Supprimer le Message Du Jour"

        command_set_welcome = u"Écrire le message de bienvenue"
        command_set_welcome_1_description = \
            u"Remplir ce formulaire avec le message de bienvenue à envoyer à" \
            u" tous les nouveaux utilisateurs"
        field_welcome = u"Message de bienvenue"

        command_delete_welcome = u"Supprimer le message de bienvenue"

        command_edit_admin = u"Editer la liste des administrateurs"
        command_edit_admin_1_description = \
            u"Remplir ce formulaire pour éditer la liste des utilisateurs qui" \
            u" ont les privilèges d'administration du service."
        field_admin_jids = u"La liste des administrateur"

        command_restart = u"Redémarrer le service"
        command_restart_1_description = \
            u"Remplir ce formulaire pour redémarrer le service."
        field_restart_delay = u"Délai avant de redémarrer le service"

        command_shutdown = u"Arrêter le service"
        command_shutdown_1_description = \
            u"Remplir ce formulaire pour arrêter le service."
        field_shutdown_delay = u"Délai avant d'arrêter le service"

        field_30_sec = u"30 secondes"
        field_60_sec = u"60 secondes"
        field_90_sec = u"90 secondes"
        field_120_sec = u"120 secondes"
        field_180_sec = u"180 secondes"
        field_240_sec = u"240 secondes"
        field_300_sec = u"300 secondes"

        welcome_message_subject = u"Bienvenue"

        account_disabled = u"Ce compte est désactivé"
        account_error = u"Ce compte est en erreur"

        help_message_subject = u"Aide"
        help_message_body = u"Pas d'aide !"

        command_last_error = u"Récupérer la dernière erreur du compte"
        field_last_error = u"Dernière erreur"
        account_no_error = u"Ce compte n'a pas d'erreur"

    class nl(en):
        # TODO: when finish, delete this line and uncomment in tests/lang.py the makeSuite(Language_nl_TestCase, 'test') line
        register_title = u"Registratie van verbindingen voor Jabber Mail"
        register_instructions = u"Instellingen voor verbinding"
        account_name = u"Accountnaam"
        account_login = u"Gebruikersnaam"
        account_password = u"Wachtwoord"
        account_password_store = u"Wachtwoord opslaan op Jabber-server?"
        account_host = u"Server"
        account_port = u"Poort"
        account_type = u"Soort account"
        account_mailbox = u"Pad naar mailbox (IMAP)"
        account_ffc_action = u"Actie bij aanwezigheid 'Chat'"
        account_online_action = u"Actie bij aanwezigheid 'Beschikbaar'"
        account_away_action = u"Actie bij aanwezigheid 'Afwezig'"
        account_xa_action = u"Actie bij aanwezigheid 'Langdurig afwezig'"
        account_dnd_action = u"Actie bij aanwezigheid 'Niet storen'"
        account_offline_action = u"Actie bij aanwezigheid 'Niet beschikbaar'"
        account_check_interval = u"Controle-interval (in minuten)"
        account_live_email_only = u"Enkel controleren op e-mails als er een" \
                                  "verbinding is met Jabber"
        action_nothing = u"Niets doen"
        action_retrieve = u"E-mail ophalen"
        action_digest = u"Samenvatting verzenden"
        update_title = u"Bijwerken van JMC"
        update_instructions = u"Verbinding '%s' aanpassen"
        connection_label = u"%s verbinding '%s'"
#        update_account_message_subject = u"Verbinding %s '%s' werd bijgewerkt"
        update_account_message_body = u"Geregistreerd met gebruikersnaam '%s'"\
                                      u"en wachtwoord '%s' op '%s'"
#         new_account_message_subject = u"Nieuwe %s verbinding '%s' aangemaakt"
        new_account_message_body = u"Geregistreerd met " \
                                   u"gebruikersnaam '%s' en wachtwoord " \
                                   u"'%s' op '%s'"
        ask_password_subject = u"Wachtwoordaanvraag"
#         ask_password_body = u"Antwoord dit bericht met het volgende " \
#                             u"wachtwoord voor de volgende account: \n" \
#                             u"\thost = %s\n" \
#                             u"\tlogin = %s\n"
        password_saved_for_session = u"Het wachtwoord zal worden bewaard " \
                                     u"tijdens uw Jabber-sessie"
        check_error_subject = u"Fout tijdens controle op e-mails."
        check_error_body = u"Fout tijdens controle op e-mails:\n\t%s"
        new_mail_subject = u"Nieuwe e-mail van %s"
        new_digest_subject = u"%i nieuwe e-mail(s)"

    class es(en):
        # TODO: when finish, delete this line and uncomment in tests/lang.py the makeSuite(Language_es_TestCase, 'test') line
        register_title = u"Registro de nueva cuenta de email"
        register_instructions = u"Inserta los datos para la nueva cuenta"
        account_name = u"Nombre para la cuenta"
        account_login = u"Usuario (login)"
        account_password = u"Contraseña"
        account_password_store = u"¿Guardar la contraseña en el servidor " \
                                 u"Jabber?"
        account_host = u"Host"
        account_port = u"Puerto"
        account_type = u"Tipo de servidor Mail"
        account_mailbox = u"Ruta del mailbox (solo para IMAP)"
        account_ffc_action = u"Acción para cuando tu estado sea " \
                             u"'Listopara hablar'"
        account_online_action = u"Acción para cuando tu estado sea 'Conectado'"
        account_away_action = u"Acción para cuando tu estado sea 'Ausente'"
        account_xa_action = u"Acción para cuando tu estado sea 'No disponible'"
        account_dnd_action = u"Acción para cuando tu estado sea 'No molestar'"
        account_offline_action = u"Acción para cuando tu estado sea " \
                                 u"'Desconectado'"
        account_check_interval = u"Intervalo para comprobar emails nuevos " \
                                 u"(en minutos)"
        account_live_email_only = u"Avisarme de emails nuevos solo cuando " \
                                  u"esté conectado"
        action_nothing = u"No hacer nada"
        action_retrieve = u"Mostrarme el email"
        action_digest = u"Enviar resúmen"
        update_title = u"Actualización de cuenta de email"
        update_instructions = u"Modifica los datos de la cuenta '%s'"
        connection_label = u"%s conexión '%s'"
#        update_account_message_subject = u"Actualizada %s conexión '%s'"
        update_account_message_body = u"Registrado con el usuario '%s' y " \
                                      u"contraseña '%s' en '%s'"
#        new_account_message_subject = u"Nueva %s conexión '%s' creada"
        new_account_message_body = u"Registrado con usuario '%s' y " \
                                   u"contraseña '%s' en '%s'"
        ask_password_subject = u"Petición de contraseña"
#         ask_password_body = u"Para avisarte de emails nuevos, contesta a " \
#                             u"este mensaje con la contraseña " \
#                             u"de la cuenta: \n" \
#                             u"\tHost = %s\n" \
#                             u"\tUsuario = %s\n"
        password_saved_for_session = u"La contraseña será guardada para " \
                                     u"esta sesión únicamente."
        check_error_subject = u"Error al revisar los emails."
        check_error_body = u"Un error apareció al revisar los emails:\n\t%s"
        new_mail_subject = u"Nuevo email en %s"
        new_digest_subject = u"%i email(s) nuevo(s)"

    class pl(en):
        # TODO: when finish, delete this line and uncomment in tests/lang.py the makeSuite(Language_pl_TestCase, 'test') line
        register_title = u"Rejestracja w komponencie E-Mail"
        register_instructions = u"Wprowadź parametry połączenia"
        account_name = u"Nazwa połączenia"
        account_login = u"Nazwa użytkownika"
        account_password = u"Hasło"
        account_password_store = u"Zachować hasło na serwerze Jabbera? "
        account_host = u"Nazwa hosta"
        account_port = u"Port"
        account_type = u"Typ serwera email"
        account_mailbox = u"Ścieżka do skrzynki odbiorczej (IMAP)"
        account_ffc_action = u"Akcja gdy status to 'Chętny do rozmowy'"
        account_online_action = u"Akcja gdy status to 'Dostępny'"
        account_away_action = u"Akcja gdy status to 'Zaraz wracam'"
        account_xa_action = u"Akcja gdy status to 'Niedostępny'"
        account_dnd_action = u"Akcja gdy status to 'Nie przeszkadzać'"
        account_offline_action = u"Akcja gdy status to 'Rozłączony'"
        account_check_interval = u"Sprawdzaj email co (w minutach)"
        account_live_email_only = u"Raportuj otrzymane emaile tylko\n gdy " \
                                  u"podłączony do Jabbera"
        action_nothing = u"Nic nie rób"
        action_retrieve = u"Pobierz emaila"
        action_digest = u"Wyślij zarys emaila"
        update_title = u"Modyfikacja połączenia z komponentem mailowym"
        update_instructions = u"Modyfikacja połączenia '%s'"
        connection_label = u"%s połączenie '%s'"
#        update_account_message_subject = u"Zmodyfikowane %s połączenie '%s'"
        update_account_message_body = u"Zarejestrowany z nazwą użytkownika " \
                                      u"'%s' i hasłem '%s' na '%s'"
#        new_account_message_subject = u"Nowe %s połączenie '%s' utworzone"
        new_account_message_body = u"Zarejestrowany z nazwą użytkownika " \
                                   u"'%s' i hasłem '%s' na '%s'"
        ask_password_subject = u"Żądanie hasła"
#         ask_password_body = u"Odpowiedz na ta wiadomosc z hasłem dla " \
#                             u"podanego konta: \n" \
#                             u"\tnazwa hosta = %s\n" \
#                             u"\tnazwa uzytkownika = %s\n"
        password_saved_for_session = u"Hasło będzie przechowywane podczas " \
                                     u"Twojej sesji Jabbera"
        check_error_subject = u"Błąd podczas sprawdzania emaili."
        check_error_body = u"Pojawił się błąd podczas sprawdzania " \
                           u"emaili:\n\t%s"
        new_mail_subject = u"Nowy email od %s"
        new_digest_subject = u"%i nowy(ch) email(i)"

    class cs(en):
        # TODO: when finish, delete this line and uncomment in tests/lang.py the makeSuite(Language_cs_TestCase, 'test') line
        register_title = u"Jabber MailNotify registrace"
        register_instructions = u"Vložte nastavení spojení"
        account_name = u"Název spojení"
        account_login = u"Přihlašovací jméno"
        account_password = u"Heslo"
        account_password_store = u"Uložit heslo na Jabber serveru?"
        account_host = u"Poštovní server"
        account_port = u"Port"
        account_type = u"Typ poštovního serveru"
        account_mailbox = u"Cesta IMAP schránky"
        account_ffc_action = u"Akce při statusu 'Ukecaný'"
        account_online_action = u"Akce při statusu 'Přítomen'"
        account_away_action = u"Akce při statusu 'Pryč'"
        account_xa_action = u"Akce při statusu 'Nepřítomen'"
        account_dnd_action = u"Akce při statusu 'Nerušit'"
        account_offline_action = u"Akce při statusu 'Odpojen'"
        account_check_interval = u"Interval kontroly pošty (v minutách)"
        account_live_email_only = u"Informuj pouze o emailech, které přišly během připojení k Jabberu"
        action_nothing = u"Nedělej nic"
        action_retrieve = u"Přijmi poštu"
        action_digest = u"Pošli upozornění na novou poštu"
        update_title = u"Jabber - aktualizace spojení k emailu"
        update_instructions = u"Aktualizace spojení '%s'"
        connection_label = u"%s spojení '%s'"
#        update_account_message_subject = u"Aktualizováno %s spojení '%s'"
        update_account_message_body = u"Registrováno s přihlašovacím jménem '%s' a " \
                                      u"heslem '%s' v '%s'"
#        new_account_message_subject = u"Nové spojení %s '%s' aktualizováno"
        new_account_message_body = u"Registrováno " \
                                   u"s přihlašovacím jménem '%s' a heslem '%s' v '%s'"
        ask_password_subject = u"Žádost o heslo"
#         ask_password_body = u"Odpovězte na tuto zprávu posláním hesla " \
#                             u"pro následující spojení: \n" \
#                             u"\thost = %s\n" \
#                             u"\tlogin = %s\n"
        password_saved_for_session = u"Heslo bude uchováno během vašeho připojení k Jabberu"
        check_error_subject = u"Chyba při kontrole emailů."
        check_error_body = u"Nějaká chyba nastala při kontrole emailů:\n\t%s"
        new_mail_subject = u"Nový email od %s"
        new_digest_subject = u"%i má nový(é) email(y)"

    class ru(en):
        # TODO: when finish, delete this line and uncomment in tests/lang.py the makeSuite(Language_ru_TestCase, 'test') line
        register_title = u"Учетные данные соединения"
        register_instructions = u"Введите данные для соединения"
        account_name = u"Имя соединения"
        account_login = u"Логин@gmail.ru"
        account_password = u"Пароль"
        account_password_store = u"Сохранять пароль на сервере?"
        account_host = u"Хост"
        account_port = u"Порт (обычно, 110)"
        account_type = u"Тип учетной записи"
        account_mailbox = u"Путь к почтовому ящику (IMAP)"
        account_ffc_action = u"Действие для состояния 'Free For Chat'"
        account_online_action = u"Действие для состояния 'Online'"
        account_away_action = u"Действие для состояния 'Away'"
        account_xa_action = u"Действие для состояния 'Not Available'"
        account_dnd_action = u"Действие для состояния 'Do not Disturb'"
        account_offline_action = u"Действие для состояния 'Offline'"
        account_check_interval = u"Интервал проверки почты (в минутах)"
        account_live_email_only = u"Сообщать о письмах только на момент подключения"
        action_nothing = u"Не делать ничего"
        action_retrieve = u"Показать почту"
        action_digest = u"Показать уведомление"
        update_title = u"Уточнение параметров"
        update_instructions = u"Изменяем соединение '%s'"
        connection_label = u"%s соединение '%s'"
#        update_account_message_subject = u"Данные для  %s обновлены '%s'"
        update_account_message_body = u"Зарегистрирован с логином '%s' и " \
                                      u"паролем '%s' на '%s'"
#        new_account_message_subject = u"Новое %s соединение '%s' создано"
        new_account_message_body = u"Перерегистрирован с " \
                                   u"логином '%s' и паролем '%s' на '%s'"
        ask_password_subject = u"Запрос пароля"
#         ask_password_body = u"Ответьте на это сообщение с паролем " \
#                             u"для следующей учетной записи: \n" \
#                             u"\tХост = %s\n" \
#                             u"\tЛогин = %s\n"
        password_saved_for_session = u"Пароль будет сохранен только на время Вашей сессии."
        check_error_subject = u"Ошибка при проверке почты."
        check_error_body = u"Возникла ошибка при проверке почты:\n\t%s"
        new_mail_subject = u"Новая почта от %s"
        new_digest_subject = u"%i новое(ые) письмо(а)"



# -*- coding: utf-8 -*-
##
## component.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Aug  9 21:04:42 2006 David Rousselie
## $Id$
##
## Copyright (C) 2006 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

"""
JCL base component
"""

__revision__ = "$Id: component.py,v 1.3 2005/09/18 20:24:07 dax Exp $"

import sys

import threading
import logging
import signal
import re
import traceback
import string

from Queue import Queue

import pyxmpp.error as error
from pyxmpp.jid import JID
from pyxmpp.jabberd.component import Component
from pyxmpp.message import Message
from pyxmpp.presence import Presence
from pyxmpp.jabber.dataforms import Form

from jcl.error import FieldError
from jcl.jabber.disco import AccountDiscoGetInfoHandler, \
     AccountTypeDiscoGetInfoHandler, RootDiscoGetItemsHandler, \
     AccountTypeDiscoGetItemsHandler
from jcl.jabber.message import PasswordMessageHandler, HelpMessageHandler
import jcl.jabber.command as command
from jcl.jabber.command import CommandDiscoGetItemsHandler, \
     CommandDiscoGetInfoHandler, JCLCommandManager, \
     CommandRootDiscoGetInfoHandler
from jcl.jabber.presence import AccountPresenceAvailableHandler, \
     RootPresenceAvailableHandler, AccountPresenceUnavailableHandler, \
     RootPresenceUnavailableHandler, AccountPresenceSubscribeHandler, \
     RootPresenceSubscribeHandler, AccountPresenceUnsubscribeHandler, \
     RootPresenceUnsubscribeHandler
from jcl.jabber.register import RootSetRegisterHandler, \
     AccountSetRegisterHandler, AccountTypeSetRegisterHandler
import jcl.model as model
from jcl.model import account
from jcl.model.account import Account, User
from jcl.lang import Lang

VERSION = "0.1"

class AccountManager(object):
    """Implement component account behavior"""

    def __init__(self, component):
        """AccountManager constructor"""
        self.__logger = logging.getLogger("jcl.jabber.JCLComponent")
        self.regexp_type = re.compile("(.*)Account$")
        self._account_classes = None
        self.account_classes = (Account,)
        self.component = component
        self.account_types = []

    def _get_account_classes(self):
        """account_classes getter"""
        return self._account_classes

    def _set_account_classes(self, account_classes):
        """account_classes setter"""
        self._account_classes = account_classes
        self.has_multiple_account_type = (len(self._account_classes) > 1)
        self.account_types = []
        for account_class in account_classes:
            match = self.regexp_type.search(account_class.__name__)
            if match is not None:
                account_type = match.group(1)
                self.account_types.append(account_type)
            else:
                self.__logger.error(account_class.__name__ +
                                    " name not well formed")
                self.account_types.append("")

    account_classes = property(_get_account_classes, _set_account_classes)

    ###### get_register handlers ######
    def account_get_register(self, info_query,
                             name,
                             from_jid,
                             account_type,
                             lang_class):
        """Handle get_register on an account.
        Return a preinitialized form"""
        info_query = info_query.make_result_response()
        account_class = self.get_account_class(account_type)
        model.db_connect()
        _account = account.get_account(unicode(from_jid.bare()), name, account_class)
        if _account is not None:
            query = info_query.new_query("jabber:iq:register")
            model.db_disconnect()
            self.generate_registration_form_init(lang_class,
                                                 _account).as_xml(query)
        else:
            model.db_disconnect()
        return [info_query]

    def _account_type_get_register(self, info_query, account_class, lang_class):
        """Handle get_register for given account_class"""
        info_query = info_query.make_result_response()
        query = info_query.new_query("jabber:iq:register")
        from_jid = info_query.get_from()
        bare_from_jid = unicode(from_jid.bare())
        self.generate_registration_form(lang_class,
                                        account_class,
                                        bare_from_jid).as_xml(query)
        return [info_query]

    def account_type_get_register(self, info_query, account_type, lang_class):
        """Handle get_register on an account_type node"""
        return self._account_type_get_register(\
            info_query,
            self.get_account_class(account_type),
            lang_class)

    def root_get_register(self, info_query, lang_class):
        """Handle get_register on root node"""
        if not self.has_multiple_account_type:
            return self._account_type_get_register(info_query,
                                                   self.account_classes[0],
                                                   lang_class)

    ###### set_register handlers ######
    def remove_all_accounts(self, user_jid):
        """Unsubscribe all accounts associated to 'user_jid' then delete
        those accounts from the DataBase"""
        model.db_connect()
        result = []
        bare_user_jid = user_jid.bare()
        for _account in account.get_accounts(bare_user_jid):
            result.extend(self.remove_account(_account, user_jid, False))
        user = account.get_user(unicode(bare_user_jid))
        if user is not None:
            user.destroySelf()
        result.append(Presence(from_jid=self.component.jid,
                               to_jid=bare_user_jid,
                               stanza_type="unsubscribe"))
        result.append(Presence(from_jid=self.component.jid,
                               to_jid=bare_user_jid,
                               stanza_type="unsubscribed"))
        model.db_disconnect()
        return result

    def remove_account_from_name(self, user_jid, name):
        _account = account.get_account(user_jid, name)
        if _account is not None:
            return self.remove_account(_account, user_jid)
        else:
            return []

    def remove_account(self, _account, user_jid, remove_user=True):
        self.__logger.debug("Deleting account: " + str(_account))
        result = []
        model.db_connect()
        bare_user_jid = user_jid.bare()
        result.append(Presence(from_jid=_account.jid,
                               to_jid=bare_user_jid,
                               stanza_type="unsubscribe"))
        result.append(Presence(from_jid=_account.jid,
                               to_jid=bare_user_jid,
                               stanza_type="unsubscribed"))
        _account.destroySelf()
        if remove_user:
            bare_user_jid = unicode(user_jid.bare())
            accounts_count = account.get_accounts_count(bare_user_jid)
            if accounts_count == 0:
                user = account.get_user(bare_user_jid)
                user.destroySelf()
        model.db_disconnect()
        return result

    def populate_account(self, _account, lang_class, x_data,
                         new_account, first_account, from_jid=None):
        """Populate given account"""
        if from_jid is None:
            from_jid = _account.user.jid
        field = None
        result = []
        for (field, field_type, field_options, field_post_func,
             field_default_func) in _account.get_register_fields():
            if field is not None:
                if field in x_data:
                    value = x_data[field].value
                else:
                    value = None
                setattr(_account, field,
                        field_post_func(value, field_default_func,
                                        unicode(from_jid.bare())))

        if hasattr(_account, "populate_handler"):
            try:
                getattr(_account, "populate_handler")()
            except Exception, exception:
                typ, value, stack = sys.exc_info()
                self.__logger.error(\
                    "Error in timer thread\n%s\n%s"
                    % (exception, "".join(traceback.format_exception
                                          (typ, value, stack, 5))))
                result.extend(self.send_error_from_account(_account,
                                                           exception))

        if first_account:
            # component subscribe user presence when registering the first
            # account
            result.append(Presence(from_jid=self.component.jid,
                                   to_jid=from_jid.bare(),
                                   stanza_type="subscribe"))
            welcome_message = self.component.get_welcome_message()
            if welcome_message is not None:
                result.append(Message(from_jid=self.component.jid,
                                      to_jid=from_jid,
                                      body=welcome_message,
                                      subject=lang_class.welcome_message_subject))
        if new_account:
            # subscribe to user presence if this is a new account
            result.append(Message(\
                    from_jid=self.component.jid,
                    to_jid=from_jid,
                    subject=_account.get_new_message_subject(lang_class),
                    body=_account.get_new_message_body(lang_class)))
            result.append(Presence(from_jid=_account.jid,
                                   to_jid=from_jid.bare(),
                                   stanza_type="subscribe"))
        else:
            result.append(Message(\
                    from_jid=self.component.jid,
                    to_jid=from_jid,
                    subject=_account.get_update_message_subject(lang_class),
                    body=_account.get_update_message_body(lang_class)))
        return result

    def update_account(self,
                       account_name,
                       from_jid,
                       account_type,
                       lang_class,
                       x_data):
        """Update account"""
        self.__logger.debug("Updating account " + account_name)
        bare_from_jid = unicode(from_jid.bare())
        _account = account.get_account(bare_from_jid,
                                       account_name)
        if _account is not None:
            return self.populate_account(_account, lang_class,
                                         x_data,
                                         new_account=False,
                                         first_account=False,
                                         from_jid=from_jid)
        else:
            account_class = self.get_account_class(account_type)
            return self.create_account(account_name,
                                       from_jid,
                                       account_class,
                                       lang_class,
                                       x_data)

    def create_account(self,
                       account_name,
                       from_jid,
                       account_class,
                       lang_class,
                       x_data):
        """Create new account from account_class"""
        bare_from_jid = unicode(from_jid.bare())
        first_account = (account.get_accounts_count(bare_from_jid) == 0)
        model.db_connect()
        user = account.get_user(bare_from_jid)
        if user is None:
            user = User(jid=bare_from_jid)
        _account = account_class(user=user,
                                 name=account_name,
                                 jid=self.get_account_jid(account_name))
        model.db_disconnect()
        try:
            return self.populate_account(_account, lang_class, x_data,
                                         new_account=True,
                                         first_account=first_account,
                                         from_jid=from_jid)
        except FieldError, field_error:
            model.db_connect()
            _account.destroySelf()
            model.db_disconnect()
            raise field_error

    def create_account_from_type(self,
                                 account_name,
                                 from_jid,
                                 account_type,
                                 lang_class,
                                 x_data):
        """Create new account from its type name"""
        account_class = self.get_account_class(account_type)
        return self.create_account(account_name,
                                   from_jid,
                                   account_class,
                                   lang_class,
                                   x_data)

    def create_default_account(self,
                               account_name,
                               bare_from_jid,
                               lang_class,
                               x_data):
        """Create new account when managing only one account type"""
        if not self.has_multiple_account_type:
            return self.create_account(account_name, bare_from_jid,
                                       self.account_classes[0],
                                       lang_class, x_data)
        else:
            return []

    ###### presence generic handlers ######
    def send_presence_all(self, presence):
        """Send presence to all account. Optimized to use only one sql
        request"""
        result = []
        model.db_connect()
        # Explicit reference to account table (clauseTables) to use
        # "user_jid" column with Account subclasses
        for user in account.get_all_users():
            result.extend(self.send_presence(self.component.jid,
                                             user.jid,
                                             presence))
        for _account in account.get_all_accounts():
            result.extend(getattr(self, "send_presence_" +
                                  presence)(_account))
        model.db_disconnect()
        return result


    def probe_all_accounts_presence(self):
        """Send presence probe to all registered accounts"""
        return self.send_presence_all("probe")

    ###### Utils methods ######
    def list_accounts(self, bare_from_jid, account_class=None,
                      account_type=""):
        """List accounts in disco_items for given _account_class and user jid"""
        if account_class is None:
            account_class = self.account_classes[0]
        if account_type is not None and account_type != "":
            resource = "/" + account_type
            account_type = account_type + "/"
        else:
            resource = ""
        model.db_connect()
        accounts = account.get_accounts(bare_from_jid, account_class)
        if accounts.count() > 0:
            for _account in accounts:
                yield (_account, resource, account_type)
        model.db_disconnect()

    def list_account_types(self, lang_class):
        """List account supported types"""
        for account_type in self.account_types:
            type_label_attr = "type_" + account_type.lower() + "_name"
            if hasattr(lang_class, type_label_attr):
                type_label = getattr(lang_class, type_label_attr)
            else:
                type_label = account_type
            yield (account_type, type_label)

    def get_account_class(self, account_type=None,
                          account_class_name=None):
        """Return account class definition from declared classes in
        account_classes from its class name"""
        if account_type is not None:
            account_class_name = account_type + "Account"
        elif account_class_name is None:
            self.__logger.error("account_type and account_class_name are None")
            return None
        self.__logger.debug("Looking for " + account_class_name)
        for _account_class in self.account_classes:
            if _account_class.__name__.lower() == account_class_name.lower():
                self.__logger.debug(account_class_name + " found")
                return _account_class
        self.__logger.debug(account_class_name + " not found")
        return None

    def generate_registration_form(self, lang_class, _account_class, bare_from_jid):
        """
        Return register form based on language and account class
        """
        reg_form = Form(title=lang_class.register_title,
                        instructions=lang_class.register_instructions)
        # "name" field is mandatory
        reg_form.add_field(field_type="text-single",
                           label=lang_class.account_name,
                           name="name",
                           required=True)

        for (field_name,
             field_type,
             field_options,
             post_func,
             default_func) in \
                _account_class.get_register_fields():
            if field_name is None:
                # TODO : Add page when empty tuple given
                pass
            else:
                lang_label_attr = "field_" + field_name
                if hasattr(lang_class, lang_label_attr):
                    label = getattr(lang_class, lang_label_attr)
                else:
                    label = field_name
                self.__logger.debug("Adding field " + field_name + " to registration form")
                field = reg_form.add_field(field_type=field_type,
                                           label=label,
                                           name=field_name,
                                           value=default_func(bare_from_jid))
                if field_options is not None:
                    for option_value in field_options:
                        lang_label_attr = "field_" + field_name + "_" + option_value
                        if hasattr(lang_class, lang_label_attr):
                            label = getattr(lang_class, lang_label_attr)
                        else:
                            label = option_value
                        field.add_option(label=label,
                                         values=[option_value])
                try:
                    post_func(None, default_func, bare_from_jid)
                except:
                    self.__logger.debug("Setting field " + field_name + " required")
                    field.required = True
        return reg_form

    def generate_registration_form_init(self, lang_class, _account):
        """
        Return register form for an existing account (update)
        """
        reg_form = self.generate_registration_form(lang_class, _account.__class__,
                                                   _account.user.jid)
        reg_form["name"].value = _account.name
        reg_form["name"].type = "hidden"
        for field in reg_form.fields:
            if hasattr(_account, field.name):
                field.value = getattr(_account, field.name)
        return reg_form

    def get_account_jid(self, name):
        """Compose account jid from account name"""
        return name + u"@" + unicode(self.component.jid)

    def send_presence_probe(self, _account):
        """Send presence probe to account's user"""
        return [Presence(from_jid=_account.jid,
                         to_jid=_account.user.jid,
                         stanza_type="probe")]

    def send_presence_unavailable(self, _account):
        """Send unavailable presence to account's user"""
        model.db_connect()
        _account.status = account.OFFLINE
        result = [Presence(from_jid=_account.jid,
                           to_jid=_account.user.jid,
                           stanza_type="unavailable")]
        model.db_disconnect()
        return result

    def send_root_presence(self, to_jid, presence_type,
                            show=None, status=None):
        result = self.send_presence(self.component.jid, to_jid,
                                    presence_type, show=show,
                                    status=status)
        result.extend(self.send_root_presence_legacy(to_jid,
                                                     presence_type,
                                                     show=show,
                                                     status=status))
        return result

    def send_presence(self, from_jid, to_jid, presence_type, status=None, show=None):
        """Send presence stanza"""
        return [Presence(from_jid=from_jid,
                         to_jid=to_jid,
                         status=status,
                         show=show,
                         stanza_type=presence_type)]

    def send_presence_available(self, _account, show, lang_class):
        """Send available presence to account's user and ask for password
        if necessary"""
        result = []
        model.db_connect()
        _account.default_lang_class = lang_class
        old_status = _account.status
        if show is None:
            _account.status = account.ONLINE
        else:
            _account.status = show
        result.append(Presence(from_jid=_account.jid,
                               to_jid=_account.user.jid,
                               status=_account.status_msg,
                               show=show,
                               stanza_type="available"))
        if hasattr(_account, 'store_password') \
            and hasattr(_account, 'password') \
            and _account.store_password == False \
            and old_status == account.OFFLINE \
            and _account.password == None :
            result.extend(self.ask_password(_account, lang_class))
        model.db_disconnect()
        return result

    def send_root_presence_legacy(self, to_jid, presence_type,
                                   status=None, show=None):
        """Send presence from legacy JID"""
        result = []
        for legacy_jid in account.get_legacy_jids(unicode(to_jid.bare())):
            result.append(Presence(from_jid=legacy_jid.jid,
                                   to_jid=to_jid,
                                   show=show,
                                   status=status,
                                   stanza_type=presence_type))
        return result

    def ask_password(self, _account, lang_class):
        """Send a Jabber message to ask for account password
        """
        result = []
        if hasattr(_account, 'waiting_password_reply') \
            and not _account.waiting_password_reply \
            and _account.status != account.OFFLINE:
            _account.waiting_password_reply = True
            result.append(Message(from_jid=_account.jid,
                                  to_jid=_account.user.jid,
                                  subject=u"[PASSWORD] " + \
                                      lang_class.ask_password_subject,
                                  body=lang_class.ask_password_body % \
                                      (_account.name)))
        return result

    def set_password(self, _account, from_jid, password, lang_class):
        """
        Set password to given account
        """
        model.db_connect()
        _account.password = password
        _account.waiting_password_reply = False
        result = [Message(from_jid=_account.jid,
                          to_jid=from_jid,
                          subject=lang_class.password_saved_for_session,
                          body=lang_class.password_saved_for_session)]
        model.db_disconnect()
        return result

    def send_error_from_account(self, _account, exception):
        """Send an error message only one time until _account.error
        has been reset to None"""
        result = []
        if _account.error == None:
            _account.error = str(exception)
            result.append(Message(from_jid=_account.jid,
                                  to_jid=_account.user.jid,
                                  stanza_type="error",
                                  subject=_account.default_lang_class.error_subject,
                                  body=_account.default_lang_class.error_body \
                                      % (exception)))
        return result

###############################################################################
# JCL implementation
###############################################################################
class JCLComponent(Component, object):
    """
    Implement default JCL component behavior:
    - Jabber register process (add, delete, update accounts)
    - Jabber presence handling
    - passwork request at login
    - Service Administration (XEP-0133)
    """

    timeout = 1

    def __init__(self,
                 jid,
                 secret,
                 server,
                 port,
                 config,
                 config_file="jmc.conf",
                 disco_category="headline",
                 disco_type="x-unknown",
                 lang=Lang(),
                 account_manager_class=AccountManager,
                 command_manager_class=JCLCommandManager):
        Component.__init__(self,
                           JID(jid),
                           secret,
                           server,
                           port,
                           disco_category=disco_category,
                           disco_type=disco_type)
        # default values
        self.name = lang.get_default_lang_class().component_name
        self.spool_dir = "."
        self.config = config
        self.config_file = config_file
        self.version = VERSION
        self.time_unit = 60
        self.queue = Queue(100)
        self.account_manager = account_manager_class(self)
        self.msg_handlers = [[PasswordMessageHandler(self),
                              HelpMessageHandler(self)]]
        self.presence_subscribe_handlers = [[AccountPresenceSubscribeHandler(self),
                                             RootPresenceSubscribeHandler(self)]]
        self.presence_unsubscribe_handlers = [[AccountPresenceUnsubscribeHandler(self),
                                               RootPresenceUnsubscribeHandler(self)]]
        self.presence_available_handlers = [[AccountPresenceAvailableHandler(self),
                                             RootPresenceAvailableHandler(self)]]
        self.presence_unavailable_handlers = [[AccountPresenceUnavailableHandler(self),
                                               RootPresenceUnavailableHandler(self)]]
        command.command_manager = command_manager_class()
        command.command_manager.component = self
        command.command_manager.account_manager = self.account_manager
        self.disco_get_items_handlers = [[RootDiscoGetItemsHandler(self),
                                          AccountTypeDiscoGetItemsHandler(self)],
                                         [CommandDiscoGetItemsHandler(self)]]
        self.disco_get_info_handlers = [[CommandRootDiscoGetInfoHandler(self),
                                         AccountDiscoGetInfoHandler(self),
                                         AccountTypeDiscoGetInfoHandler(self)],
                                        [CommandDiscoGetInfoHandler(self)]]
        self.set_register_handlers = [[RootSetRegisterHandler(self),
                                       AccountSetRegisterHandler(self),
                                       AccountTypeSetRegisterHandler(self)]]

        self.__logger = logging.getLogger("jcl.jabber.JCLComponent")
        self.lang = lang
        self.running = False
        self.wait_event = threading.Event()
        self._restart = False

        signal.signal(signal.SIGINT, self.signal_handler)
        signal.signal(signal.SIGTERM, self.signal_handler)

    def run(self):
        """
        Main loop
        Connect to Jabber server
        Start timer thread
        Call Component main loop
        Clean up when shutting down JCLcomponent
        """
        self.spool_dir += "/" + unicode(self.jid)
        self.running = True
        self.connect()
        timer_thread = threading.Thread(target=self.time_handler,
                                        name="TimerThread")
        timer_thread.start()
        try:
            while (self.running and self.stream
                   and not self.stream.eof
                   and self.stream.socket is not None):
                self.stream.loop_iter(JCLComponent.timeout)
                if self.queue.qsize():
                    raise self.queue.get(0)
        finally:
            self.running = False
            if self.stream and not self.stream.eof \
                   and self.stream.socket is not None:
                presences = self.account_manager.send_presence_all("unavailable")
                self.send_stanzas(presences)
            self.wait_event.set()
            timer_thread.join(JCLComponent.timeout)
            self.disconnect()
            self.__logger.debug("Exitting normally")
        return self._restart

    def _get_restart(self):
        return self._restart

    def _set_restart(self, __restart):
        self.running = not __restart
        self._restart = __restart

    restart = property(_get_restart, _set_restart)

    ###########################################################################
    # Handlers
    ###########################################################################
    def time_handler(self):
        """
        Timer thread handler
        """
        self.__logger.info("Timer thread started...")
        try:
            while (self.running and self.stream
                   and not self.stream.eof
                   and self.stream.socket is not None):
                self.wait_event.wait(self.time_unit)
                self.handle_tick()
                self.__logger.debug(".")
        except Exception, exception:
            type, value, stack = sys.exc_info()
            self.__logger.error("Error in timer thread\n%s\n%s"
                                % (exception, "".join(traceback.format_exception
                                                      (type, value, stack, 5))))
            self.queue.put(exception)
        self.__logger.info("Timer thread terminated...")

    def authenticated(self):
        """
        Override authenticated Component event handler
        Register event handlers
        Probe for every accounts registered
        """
        self.__logger.debug("AUTHENTICATED")
        Component.authenticated(self)
        self.stream.set_iq_get_handler("query", "jabber:iq:version",
                                       self.handle_get_version)
        self.stream.set_iq_get_handler("query", "jabber:iq:register",
                                       self.handle_get_register)
        self.stream.set_iq_set_handler("query", "jabber:iq:register",
                                       self.handle_set_register)
        self.stream.set_iq_get_handler("query", "jabber:iq:gateway",
                                       self.handle_get_gateway)
        self.stream.set_iq_set_handler("query", "jabber:iq:gateway",
                                       self.handle_set_gateway)

        self.stream.set_iq_set_handler("command", command.COMMAND_NS,
                                       self.handle_command)

        self.stream.set_presence_handler("available",
                                         self.handle_presence_available)

        self.stream.set_presence_handler("probe",
                                         self.handle_presence_available)

        self.stream.set_presence_handler("unavailable",
                                         self.handle_presence_unavailable)

        self.stream.set_presence_handler("unsubscribe",
                                         self.handle_presence_unsubscribe)
        self.stream.set_presence_handler("unsubscribed",
                                         self.handle_presence_unsubscribed)
        self.stream.set_presence_handler("subscribe",
                                         self.handle_presence_subscribe)
        self.stream.set_presence_handler("subscribed",
                                         self.handle_presence_subscribed)

        self.stream.set_message_handler("normal",
                                        self.handle_message)
        self.send_stanzas(self.account_manager.probe_all_accounts_presence())

    def signal_handler(self, signum, frame):
        """Stop method handler
        """
        self.__logger.debug("Signal %i received, shutting down..." % (signum,))
        self.running = False

    def send_stanzas(self, stanzas):
        """Send given stanza list"""
        self.__logger.debug("Sending responses: " + str(stanzas))
        if stanzas is not None:
            for stanza in stanzas:
                self.stream.send(stanza)

    def apply_behavior(self, info_query,
                       account_handler,
                       account_type_handler,
                       root_handler,
                       send_result=False):
        """Apply given handler depending on info_query receiver"""
        from_jid = info_query.get_from()
        to_jid = info_query.get_to()
        name = to_jid.node
        account_type = to_jid.resource
        lang_class = self.lang.get_lang_class_from_node(info_query.get_node())
        # * root
        # |-* account_type1
        # | |-* account1
        # | |-* account2
        # |-* account_type2
        #   |-* account3
        #   |-* account4
        if name is not None: # account
            self.__logger.debug("Applying behavior on account " + name)
            result = account_handler(name, from_jid, account_type or "",
                                     lang_class)
        elif account_type is None: # root
            self.__logger.debug("Applying behavior on root node")
            result = root_handler(name, from_jid, "",
                                  lang_class)
        else: # account type
            self.__logger.debug("Applying behavior on account type " +
                                account_type)
            result = account_type_handler(name, from_jid, account_type,
                                          lang_class)
        if send_result:
            self.send_stanzas(result)
        return result

    def apply_registered_behavior(self, handlers, stanza,
                                  apply_filter_func=None,
                                  apply_handle_func=None,
                                  send_result=True):
        """
        Execute handler if their filter method does not return None
        `handlers` is a list of handler groups which is a list of groups.
        Only one successfull handler (its filter and its handler methods
        does not return None) per group is executed. When a handler is
        executed successfully the next group is processed.
        """
        result = []
        lang_class = self.lang.get_lang_class_from_node(stanza.get_node())
        for handler_group in handlers:
            for handler in handler_group:
                try:
                    self.__logger.debug("Applying filter " + repr(handler))
                    if apply_filter_func is not None:
                        data = apply_filter_func(handler.filter, stanza, lang_class)
                    else:
                        data = handler.filter(stanza, lang_class)
                    if data is not None and data != False:
                        self.__logger.debug("Applying handler " + repr(handler))
                        if apply_handle_func is not None:
                            handler_result = apply_handle_func(handler.handle,
                                                               stanza,
                                                               lang_class,
                                                               data,
                                                               result)
                        else:
                            handler_result = handler.handle(stanza, lang_class,
                                                            data)
                        if handler_result is not None:
                            result += handler_result
                            break
                except Exception, e:
                    error_type, value, stack = sys.exc_info()
                    self.__logger.error("Error with handler " + str(handler) +
                                        " with " + str(stanza) + "\n%s\n%s"
                                        % (e, "".join(traceback.format_exception
                                                      (error_type, value, stack, 5))))
                    result += [Message(from_jid=stanza.get_to(),
                                       to_jid=stanza.get_from(),
                                       stanza_type="error",
                                       subject=lang_class.error_subject,
                                       body=lang_class.error_body % (e))]
        if send_result:
            self.send_stanzas(result)
        return result

    def handle_get_gateway(self, info_query):
        """Handle IQ-get "jabber:iq:gateway" requests.
        Return prompt and description.
        """
        self.__logger.debug("GET_GATEWAY")
        info_query = info_query.make_result_response()
        lang_class = self.lang.get_lang_class_from_node(info_query.get_node())
        query = info_query.new_query("jabber:iq:gateway")
        query.newTextChild(query.ns(), "desc", lang_class.get_gateway_desc)
        query.newTextChild(query.ns(), "prompt", lang_class.get_gateway_prompt)
        self.stream.send(info_query)
        return 1

    def handle_set_gateway(self, info_query):
        """
        Handle IQ-set "jabber:iq:gateway" requests.
        Return well formed JID from legacy ID.
        """
        self.__logger.debug("SET_GATEWAY")
        prompt_nodes = info_query.xpath_eval("jig:query/jig:prompt",
                                             {"jig" : "jabber:iq:gateway"})
        jid = prompt_nodes[0].content.replace("@", "%") \
            + "@" + unicode(self.jid)
        info_query = info_query.make_result_response()
        query = info_query.new_query("jabber:iq:gateway")
        query.newTextChild(query.ns(), "jid", jid)
        # XEP-0100 - section 6: should be <jid> but PSI only work with <prompt>
        query.newTextChild(query.ns(), "prompt", jid)
        self.stream.send(info_query)
        return 1

    def disco_get_info(self, node, info_query):
        """
        Discovery get info handler
        """
        result = self.apply_registered_behavior(\
            self.disco_get_info_handlers,
            info_query,
            lambda filter_func, stanza, lang_class: \
                filter_func(stanza, lang_class, node),
            lambda handle_func, stanza, lang_class, data, result: \
                handle_func(stanza, lang_class, node, result, data),
            send_result=False)
        if len(result) > 0:
            return result[0]
        else:
            return None

    def disco_get_items(self, node, info_query):
        """
        Discovery get nested nodes handler
        """
        result = self.apply_registered_behavior(\
            self.disco_get_items_handlers,
            info_query,
            lambda filter_func, stanza, lang_class: \
                filter_func(stanza, lang_class, node),
            lambda handle_func, stanza, lang_class, data, result: \
                handle_func(stanza, lang_class, node, result, data),
            send_result=False)
        if len(result) > 0:
            return result[0]
        else:
            return None

    def handle_get_version(self, info_query):
        """Get Version handler
        """
        self.__logger.debug("GET_VERSION")
        info_query = info_query.make_result_response()
        query = info_query.new_query("jabber:iq:version")
        query.newTextChild(query.ns(), "name", self.name)
        query.newTextChild(query.ns(), "version", self.version)
        self.stream.send(info_query)
        return 1

    def handle_get_register(self, info_query):
        """Send back register form to user
        see node structure in disco_get_items()
        """
        self.__logger.debug("GET_REGISTER")
        self.apply_behavior(\
            info_query,
            lambda name, from_jid, account_type, lang_class: \
                self.account_manager.account_get_register(info_query,
                                                          name,
                                                          from_jid,
                                                          account_type,
                                                          lang_class),
            lambda name, from_jid, account_type, lang_class: \
                self.account_manager.account_type_get_register(info_query,
                                                               account_type,
                                                               lang_class),
            lambda name, from_jid, account_type, lang_class: \
                self.account_manager.root_get_register(info_query,
                                                       lang_class),
            send_result=True)

    def handle_set_register(self, info_query):
        """Handle user registration response
        """
        self.__logger.debug("SET_REGISTER")
        lang_class = \
            self.lang.get_lang_class_from_node(info_query.get_node())
        from_jid = info_query.get_from()
        remove = info_query.xpath_eval("r:query/r:remove",
                                       {"r" : "jabber:iq:register"})
        if remove:
            result = self.account_manager.remove_all_accounts(from_jid)
            self.send_stanzas(result)
            return 1

        x_node = info_query.xpath_eval("jir:query/jxd:x",
                                       {"jir" : "jabber:iq:register",
                                        "jxd" : "jabber:x:data"})[0]
        x_data = Form(x_node)
        return self.apply_registered_behavior(\
            self.set_register_handlers,
            info_query,
            apply_filter_func=lambda filter_func, stanza, lang_class: \
                filter_func(stanza, lang_class, x_data),
            apply_handle_func=lambda handle_func, stanza, lang_class, data, result: \
                handle_func(stanza, lang_class, data, x_data))

    def handle_presence_available(self, stanza):
        """Handle presence availability
        if presence sent to the component ('if not name'), presence is sent to
        all accounts for current user. Otherwise, send presence from current account.
        """
        result = self.apply_registered_behavior(self.presence_available_handlers,
                                                stanza)
        return result


    def handle_presence_unavailable(self, stanza):
        """Handle presence unavailability
        """
        self.__logger.debug("PRESENCE_UNAVAILABLE")
        result = self.apply_registered_behavior(self.presence_unavailable_handlers,
                                                stanza)
        return result

    def handle_presence_subscribe(self, stanza):
        """Handle subscribe presence from user
        """
        self.__logger.debug("PRESENCE_SUBSCRIBE")
        result = self.apply_registered_behavior(self.presence_subscribe_handlers,
                                                stanza)
        return result

    def handle_presence_subscribed(self, stanza):
        """Handle subscribed presence from user
        """
        self.__logger.debug("PRESENCE_SUBSCRIBED")
        return 1

    def handle_presence_unsubscribe(self, stanza):
        """Handle unsubscribe presence from user
        """
        self.__logger.debug("PRESENCE_UNSUBSCRIBE")
        result = self.apply_registered_behavior(self.presence_unsubscribe_handlers,
                                                stanza)
        return result

    def handle_presence_unsubscribed(self, stanza):
        """Handle unsubscribed presence from user
        """
        self.__logger.debug("PRESENCE_UNSUBSCRIBED")
        presence = Presence(from_jid=stanza.get_to(),
                            to_jid=stanza.get_from(),
                            stanza_type="unavailable")
        self.stream.send(presence)
        return 1

    def handle_message(self, message):
        """Handle new message
        Handle password response message
        """
        self.__logger.debug("MESSAGE: " + message.get_body())
        self.apply_registered_behavior(self.msg_handlers, message)
        return 1

    def handle_command(self, info_query):
        """
        Handle command IQ
        """
        self.__logger.debug("COMMAND")

        _command = info_query.xpath_eval("c:command",
                                         {"c": command.COMMAND_NS})[0]
        command_node = _command.prop("node")
        action = _command.prop("action")
        if action is None:
            action = "execute"
        try:
            result = command.command_manager.apply_command_action(info_query,
                                                                  command_node,
                                                                  action)
            self.send_stanzas(result)
        except:
            type, value, stack = sys.exc_info()
            self.__logger.error("Error in command " + str(command_node) +
                                " with " + str(info_query) + "\n%s"
                                % ("".join(traceback.format_exception
                                           (type, value, stack, 5))))
            print("Error in command " + str(command_node) +
                                " with " + str(info_query) + "\n%s"
                                % ("".join(traceback.format_exception
                                           (type, value, stack, 5))))
        return 1

    ###########################################################################
    # Utils
    ###########################################################################
    def send_error(self, _account, exception):
        """ """
        self.send_stanzas(self.account_manager.send_error_from_account(_account, exception))
        type, value, stack = sys.exc_info()
        self.__logger.debug("Error: %s\n%s"
                            % (exception, "".join(traceback.format_exception
                                                  (type, value, stack, 5))))

    def get_config_parameter(self, section, parameter):
        if self.config is not None \
               and self.config.has_option(section, parameter):
            return self.config.get(section, parameter)
        else:
            return None

    def set_config_parameter(self, section, parameter, value):
        if not self.config.has_section(section):
            self.config.add_section(section)
        self.config.set(section, parameter, value)
        configFile = open(self.config_file, "w")
        self.config.write(configFile)
        configFile.close()

    def del_config_parameter(self, section, parameter):
        if self.config.has_section(section) \
           and self.config.has_option(section, parameter):
            self.config.remove_option(section, parameter)
            configFile = open(self.config_file, "w")
            self.config.write(configFile)
            configFile.close()

    def get_motd(self):
        return self.get_config_parameter("component", "motd")

    def set_motd(self, motd):
        self.set_config_parameter("component", "motd", motd)

    def del_motd(self):
        self.del_config_parameter("component", "motd")

    def get_admins(self):
        admins_str = self.get_config_parameter("component",
                                               "admins")
        if admins_str is not None:
            return map(string.strip,
                       admins_str.split(','))
        else:
            return []

    def set_admins(self, admins):
        self.set_config_parameter("component", "admins", ",".join(admins))

    def is_admin(self, jid):
        if isinstance(jid, JID):
            jid_str = unicode(jid.bare())
        else:
            jid_str = unicode(jid)
        return jid_str in self.get_admins()

    def get_welcome_message(self):
        return self.get_config_parameter("component", "welcome_message")

    def set_welcome_message(self, welcome_message):
        self.set_config_parameter("component", "welcome_message",
                                  welcome_message)

    def del_welcome_message(self):
        self.del_config_parameter("component", "welcome_message")


    ###########################################################################
    # Virtual methods
    ###########################################################################
    def handle_tick(self):
        """Virtual method
        Called regularly
        """
        raise NotImplementedError




##
## register.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Jul 18 21:32:51 2007 David Rousselie
## $Id$
## 
## Copyright (C) 2007 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import logging
import sys
import traceback

import pyxmpp.error as error

from jcl.error import FieldError
import jcl.jabber as jabber

class SetRegisterHandler(object):
    """Handle disco get items requests"""

    def __init__(self, component):
        self.component = component
        self.account_manager = component.account_manager
        self.__logger = logging.getLogger("jcl.jabber.SetRegisterHandler")

    def filter(self, info_query, lang_class, x_data):
        """Filter requests to be handled"""
        return False

    def handle(self, info_query, lang_class, data, x_data):
        """Handle disco get items request"""
        return None

    def handle_error(self, field_error, info_query, lang_class):
        type, value, stack = sys.exc_info()
        self.__logger.error("Error while populating account: %s\n%s" %
                            (field_error, "".join(traceback.format_exception
                                                (type, value, stack, 5))))
        iq_error = info_query.make_error_response("not-acceptable")
        text = iq_error.get_error().xmlnode.newTextChild(\
            None,
            "text",
            lang_class.mandatory_field % (field_error.field))
        text.setNs(text.newNs(error.STANZA_ERROR_NS, None))
        return [iq_error]

class RootSetRegisterHandler(SetRegisterHandler):

    def __init__(self, component):
        SetRegisterHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.RootSetRegisterHandler")

    def filter(self, stanza, lang_class, x_data):
        """
        """
        return jabber.root_filter(self, stanza, lang_class)

    def handle(self, info_query, lang_class, data, x_data):
        """
        """
        self.__logger.debug("root_set_register")
        _account = None
        if not "name" in x_data or x_data["name"].value == "":
            return self.handle_error(FieldError("name", ""),
                                     info_query, lang_class)
        try:
            info_queries = self.account_manager.create_default_account(\
                    x_data["name"].value,
                    info_query.get_from(),
                    lang_class,
                    x_data)
            info_queries.insert(0, info_query.make_result_response())
        except FieldError, field_error:
            return self.handle_error(field_error,
                                     info_query, lang_class)
        return info_queries

class AccountSetRegisterHandler(SetRegisterHandler):

    def __init__(self, component):
        SetRegisterHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.AccountSetRegisterHandler")

    def filter(self, stanza, lang_class, x_data):
        """
        """
        return jabber.account_filter(self, stanza, lang_class)

    def handle(self, info_query, lang_class, data, x_data):
        """
        """
        self.__logger.debug("account_set_register")
        _account = None
        resource = info_query.get_to().resource
        if resource is not None:
            account_type = resource.split("/")[0]
        else:
            account_type = ""
        try:
            info_queries = self.account_manager.update_account(\
                x_data["name"].value,
                info_query.get_from(),
                account_type,
                lang_class,
                x_data)
            info_queries.insert(0, info_query.make_result_response())
        except FieldError, field_error:
            return self.handle_error(field_error,
                                     info_query, lang_class)
        return info_queries

class AccountTypeSetRegisterHandler(SetRegisterHandler):

    def __init__(self, component):
        SetRegisterHandler.__init__(self, component)
        self.__logger = logging.getLogger("jcl.jabber.AccountTypeSetRegisterHandler")

    def filter(self, stanza, lang_class, x_data):
        """
        """
        return jabber.account_type_filter(self, stanza, lang_class)

    def handle(self, info_query, lang_class, data, x_data):
        """
        """
        self.__logger.debug("account_type_set_register")
        account_type = data
        _account = None
        try:
            info_queries = self.account_manager.create_account_from_type(\
                x_data["name"].value,
                info_query.get_from(),
                account_type,
                lang_class,
                x_data)
            info_queries.insert(0, info_query.make_result_response())
        except FieldError, field_error:
            return self.handle_error(field_error,
                                     info_query, lang_class)
        return info_queries



# -*- coding: utf-8 -*-
##
## component.py
## Login : David Rousselie <dax@happycoders.org>
## Started on  Wed Jun 27 21:42:47 2007 David Rousselie
## $Id$
##
## Copyright (C) 2006 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

from pyxmpp.presence import Presence
from pyxmpp.message import Message

from jcl.jabber import Handler
from jcl.model import account
import jcl.jabber as jabber

class DefaultPresenceHandler(Handler):
    """Handle presence"""

    def handle(self, stanza, lang_class, data):
        """Return same presence as receive one"""
        to_jid = stanza.get_to()
        from_jid = stanza.get_from()
        stanza.set_to(from_jid)
        stanza.set_from(to_jid)
        return [stanza]

class DefaultSubscribeHandler(Handler):
    """Return default response to subscribe queries"""

    def handle(self, stanza, lang_class, data):
        """Create subscribe response"""
        result = []
        result.append(Presence(from_jid=stanza.get_to(),
                               to_jid=stanza.get_from(),
                               stanza_type="subscribe"))
        result.append(Presence(from_jid=stanza.get_to(),
                               to_jid=stanza.get_from(),
                               stanza_type="subscribed"))
        return result

class DefaultUnsubscribeHandler(Handler):
    """Return default response to subscribe queries"""

    def handle(self, stanza, lang_class, data):
        """Create subscribe response"""
        result = []
        result.append(Presence(from_jid=stanza.get_to(),
                               to_jid=stanza.get_from(),
                               stanza_type="unsubscribe"))
        result.append(Presence(from_jid=stanza.get_to(),
                               to_jid=stanza.get_from(),
                               stanza_type="unsubscribed"))
        return result

class AccountPresenceHandler(Handler):
    filter = jabber.get_account_filter
    
    def get_account_presence(self, stanza, lang_class, _account):
        raise NotImplemented

    def handle(self, stanza, lang_class, data):
        """Handle presence sent to an account JID"""
        result = []
        _account = data
        result.extend(self.get_account_presence(stanza, lang_class, _account))
        return result

class AccountPresenceAvailableHandler(AccountPresenceHandler):
    def get_account_presence(self, stanza, lang_class, _account):
        show_status = account.ONLINE
        if _account.error is not None:
            show_status = account.DND
        elif not _account.enabled:
            show_status = account.XA
        return self.component.account_manager.send_presence_available(\
            _account,
            show_status,
            lang_class)

class RootPresenceHandler(AccountPresenceHandler):
    filter = jabber.get_accounts_root_filter

    def get_root_presence(self, stanza, lang_class, nb_accounts):
        raise NotImplemented

    def handle(self, stanza, lang_class, data):
        """handle presence sent to component JID"""
        result = []
        accounts_length = 0
        accounts = data
        for _account in accounts:
            accounts_length += 1
            result.extend(self.get_account_presence(stanza, lang_class, _account))
        if accounts_length > 0:
            result.extend(self.get_root_presence(stanza, lang_class, accounts_length))
        return result
        
class RootPresenceAvailableHandler(RootPresenceHandler, AccountPresenceAvailableHandler):
    def get_root_presence(self, stanza, lang_class, nb_accounts):
        from_jid = stanza.get_from()
        result = self.component.account_manager.send_root_presence(from_jid,
                                                                   "available",
                                                                   "online",
                                                                   str(nb_accounts) +
                                                                   lang_class.message_status)
        user = account.get_user(unicode(from_jid.bare()))
        if not user.has_received_motd:
            user.has_received_motd = True
            motd = self.component.get_motd()
            if motd is not None:
                result.append(Message(from_jid=self.component.jid,
                                      to_jid=from_jid,
                                      body=motd))
        return result
        

class AccountPresenceUnavailableHandler(AccountPresenceHandler):
    def get_account_presence(self, stanza, lang_class, _account):
        return self.component.account_manager.send_presence_unavailable(_account)

class RootPresenceUnavailableHandler(RootPresenceHandler, AccountPresenceUnavailableHandler):    
    def get_root_presence(self, stanza, lang_class, nb_accounts):
        return self.component.account_manager.send_root_presence(stanza.get_from(),
                                                                 "unavailable")

class AccountPresenceSubscribeHandler(Handler):
    """"""

    filter = jabber.get_account_filter

    def handle(self, stanza, lang_class, data):
        """Handle \"subscribe\" iq sent to an account JID"""
        return [stanza.make_accept_response()]

class RootPresenceSubscribeHandler(AccountPresenceSubscribeHandler):
    """"""

    filter = jabber.get_accounts_root_filter

    def handle(self, stanza, lang_class, data):
        """Handle \"subscribe\" iq sent to component JID"""
        if list(data) != []:
            return AccountPresenceSubscribeHandler.handle(self, stanza,
                                                          lang_class, None)
        else:
            return None

class AccountPresenceUnsubscribeHandler(Handler):
    """"""

    filter = jabber.get_account_filter

    def handle(self, stanza, lang_class, data):
        """Handle \"unsubscribe\" iq sent to account JID"""
        from_jid = stanza.get_from()
        _account = data
        return self.component.account_manager.remove_account(_account, from_jid)

class RootPresenceUnsubscribeHandler(Handler):
    """"""

    filter = jabber.get_accounts_root_filter

    def handle(self, stanza, lang_class, data):
        """Handle \"unsubscribe\" iq sent to account JID"""
        from_jid = stanza.get_from()
        _account = data
        return self.component.account_manager.remove_all_accounts(from_jid)



##
## setup.py
## Login : <dax@happycoders.org>
## Started on  Tue Apr 17 21:12:33 2007 David Rousselie
## $Id$
## 
## Copyright (C) 2007 David Rousselie
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

from setuptools import setup, find_packages

setup(name='jcl',
      version='0.1b1',
      description='Jabber Component Library',
      author='David Rousselie',
      author_email='dax@happycoders.org',
      license="LGPL",
      keywords="jabber component",
      url='http://people.happycoders.org/dax/projects/jcl',
      classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Programming Language :: Python",
        "Topic :: Communications",
        "Topic :: Communications :: Chat",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      package_dir={'': 'src'},
      packages=find_packages('src', exclude=["*.tests",
                                             "*.tests.*",
                                             "tests.*",
                                             "tests"]),
      test_suite='jcl.tests.suite',
      install_requires=['SQLObject>=0.8', 'pyxmpp>=1.0', 'pysqlite>=2.0'])
